@@ -4,36 +4,30 @@ use v5.12.0;
 use warnings;
 
 use Carp qw(croak);
-use B::Hooks::EndOfScope;
 
 use XSLoader;
 BEGIN {
-	our $VERSION = '0.03';
-	XSLoader::load __PACKAGE__, $VERSION;
+    our $VERSION = '0.04';
+    XSLoader::load __PACKAGE__, $VERSION;
 }
 
-# all shall burn
-our @meta;
-
 sub define {
-	my ($kw, $sub) = @_;
-	$kw =~ /^\p{XIDS}\p{XIDC}*\z/ or croak "'$kw' doesn't look like an identifier";
-	ref($sub) eq 'CODE' or croak "'$sub' doesn't look like a coderef";
+    my ($kw, $sub) = @_;
+    $kw =~ /^\p{XIDS}\p{XIDC}*\z/ or croak "'$kw' doesn't look like an identifier";
+    ref($sub) eq 'CODE' or croak "'$sub' doesn't look like a coderef";
 
-	my $n = @meta;
-	push @meta, $sub;
-
-	$^H{+HINTK_KEYWORDS} .= " $kw:$n";
-	on_scope_end {
-		delete $meta[$n];
-	};
+    my %keywords = %{$^H{+HINTK_KEYWORDS} // {}};
+    $keywords{$kw} = $sub;
+    $^H{+HINTK_KEYWORDS} = \%keywords;
 }
 
 sub undefine {
-	my ($kw) = @_;
-	$kw =~ /^\p{XIDS}\p{XIDC}*\z/ or croak "'$kw' doesn't look like an identifier";
+    my ($kw) = @_;
+    $kw =~ /^\p{XIDS}\p{XIDC}*\z/ or croak "'$kw' doesn't look like an identifier";
 
-	$^H{+HINTK_KEYWORDS} .= " $kw:-";
+    my %keywords = %{$^H{+HINTK_KEYWORDS} // {}};
+    delete $keywords{$kw};
+    $^H{+HINTK_KEYWORDS} = \%keywords;
 }
 
 'ok'
@@ -42,6 +36,8 @@ __END__
 
 =encoding UTF-8
 
+=for highlighter language=perl
+
 =head1 NAME
 
 Keyword::Simple - define new keywords in pure Perl
@@ -127,15 +123,61 @@ and the rest of the file is unchanged.
 This also means your new keywords can only occur at the beginning of a
 statement, not embedded in an expression.
 
+Keywords in the replacement part of a C<s//.../e> substitution aren't handled
+correctly and break parsing.
+
 There are barely any tests.
 
+=begin :README
+
+=head1 INSTALLATION
+
+To download and install this module, use your favorite CPAN client, e.g.
+L<C<cpan>|cpan>:
+
+=for highlighter language=sh
+
+    cpan Keyword::Simple
+
+Or L<C<cpanm>|cpanm>:
+
+    cpanm Keyword::Simple
+
+To do it manually, run the following commands (after downloading and unpacking
+the tarball):
+
+    perl Makefile.PL
+    make
+    make test
+    make install
+
+=end :README
+
+=head1 SUPPORT AND DOCUMENTATION
+
+After installing, you can find documentation for this module with the
+L<C<perldoc>|perldoc> command.
+
+=for highlighter language=sh
+
+    perldoc Keyword::Simple
+
+You can also look for information at
+L<https://metacpan.org/pod/Keyword::Simple>.
+
+To see a list of open bugs, visit
+L<https://rt.cpan.org/Public/Dist/Display.html?Name=Keyword-Simple>.
+
+To report a new bug, send an email to
+C<bug-Keyword-Simple [at] rt.cpan.org>.
+
 =head1 AUTHOR
 
 Lukas Mai, C<< <l.mai at web.de> >>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2012, 2013 Lukas Mai.
+Copyright (C) 2012, 2013 Lukas Mai.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
